# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from openapi_client.configuration import Configuration


class GithubOrganization(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'avatar_url': 'str',
        'billing_email': 'str',
        'blog': 'str',
        'collaborators': 'int',
        'company': 'str',
        'created_at': 'datetime',
        'default_repository_permission': 'str',
        'default_repository_settings': 'str',
        'description': 'str',
        'disk_usage': 'int',
        'email': 'str',
        'events_url': 'str',
        'followers': 'int',
        'following': 'int',
        'has_organization_projects': 'bool',
        'has_repository_projects': 'bool',
        'hooks_url': 'str',
        'html_url': 'str',
        'id': 'int',
        'is_verified': 'bool',
        'issues_url': 'str',
        'location': 'str',
        'login': 'str',
        'members_allowed_repository_creation_type': 'str',
        'members_can_create_internal_repositories': 'bool',
        'members_can_create_pages': 'bool',
        'members_can_create_private_pages': 'bool',
        'members_can_create_private_repositories': 'bool',
        'members_can_create_public_pages': 'bool',
        'members_can_create_public_repositories': 'bool',
        'members_can_create_repositories': 'bool',
        'members_url': 'str',
        'name': 'str',
        'node_id': 'str',
        'owned_private_repos': 'int',
        'plan': 'GithubPlan',
        'private_gists': 'int',
        'public_gists': 'int',
        'public_members_url': 'str',
        'public_repos': 'int',
        'repos_url': 'str',
        'total_private_repos': 'int',
        'twitter_username': 'str',
        'two_factor_requirement_enabled': 'bool',
        'type': 'str',
        'updated_at': 'datetime',
        'url': 'str'
    }

    attribute_map = {
        'avatar_url': 'avatar_url',
        'billing_email': 'billing_email',
        'blog': 'blog',
        'collaborators': 'collaborators',
        'company': 'company',
        'created_at': 'created_at',
        'default_repository_permission': 'default_repository_permission',
        'default_repository_settings': 'default_repository_settings',
        'description': 'description',
        'disk_usage': 'disk_usage',
        'email': 'email',
        'events_url': 'events_url',
        'followers': 'followers',
        'following': 'following',
        'has_organization_projects': 'has_organization_projects',
        'has_repository_projects': 'has_repository_projects',
        'hooks_url': 'hooks_url',
        'html_url': 'html_url',
        'id': 'id',
        'is_verified': 'is_verified',
        'issues_url': 'issues_url',
        'location': 'location',
        'login': 'login',
        'members_allowed_repository_creation_type': 'members_allowed_repository_creation_type',
        'members_can_create_internal_repositories': 'members_can_create_internal_repositories',
        'members_can_create_pages': 'members_can_create_pages',
        'members_can_create_private_pages': 'members_can_create_private_pages',
        'members_can_create_private_repositories': 'members_can_create_private_repositories',
        'members_can_create_public_pages': 'members_can_create_public_pages',
        'members_can_create_public_repositories': 'members_can_create_public_repositories',
        'members_can_create_repositories': 'members_can_create_repositories',
        'members_url': 'members_url',
        'name': 'name',
        'node_id': 'node_id',
        'owned_private_repos': 'owned_private_repos',
        'plan': 'plan',
        'private_gists': 'private_gists',
        'public_gists': 'public_gists',
        'public_members_url': 'public_members_url',
        'public_repos': 'public_repos',
        'repos_url': 'repos_url',
        'total_private_repos': 'total_private_repos',
        'twitter_username': 'twitter_username',
        'two_factor_requirement_enabled': 'two_factor_requirement_enabled',
        'type': 'type',
        'updated_at': 'updated_at',
        'url': 'url'
    }

    def __init__(self, avatar_url=None, billing_email=None, blog=None, collaborators=None, company=None, created_at=None, default_repository_permission=None, default_repository_settings=None, description=None, disk_usage=None, email=None, events_url=None, followers=None, following=None, has_organization_projects=None, has_repository_projects=None, hooks_url=None, html_url=None, id=None, is_verified=None, issues_url=None, location=None, login=None, members_allowed_repository_creation_type=None, members_can_create_internal_repositories=None, members_can_create_pages=None, members_can_create_private_pages=None, members_can_create_private_repositories=None, members_can_create_public_pages=None, members_can_create_public_repositories=None, members_can_create_repositories=None, members_url=None, name=None, node_id=None, owned_private_repos=None, plan=None, private_gists=None, public_gists=None, public_members_url=None, public_repos=None, repos_url=None, total_private_repos=None, twitter_username=None, two_factor_requirement_enabled=None, type=None, updated_at=None, url=None, local_vars_configuration=None):  # noqa: E501
        """GithubOrganization - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._avatar_url = None
        self._billing_email = None
        self._blog = None
        self._collaborators = None
        self._company = None
        self._created_at = None
        self._default_repository_permission = None
        self._default_repository_settings = None
        self._description = None
        self._disk_usage = None
        self._email = None
        self._events_url = None
        self._followers = None
        self._following = None
        self._has_organization_projects = None
        self._has_repository_projects = None
        self._hooks_url = None
        self._html_url = None
        self._id = None
        self._is_verified = None
        self._issues_url = None
        self._location = None
        self._login = None
        self._members_allowed_repository_creation_type = None
        self._members_can_create_internal_repositories = None
        self._members_can_create_pages = None
        self._members_can_create_private_pages = None
        self._members_can_create_private_repositories = None
        self._members_can_create_public_pages = None
        self._members_can_create_public_repositories = None
        self._members_can_create_repositories = None
        self._members_url = None
        self._name = None
        self._node_id = None
        self._owned_private_repos = None
        self._plan = None
        self._private_gists = None
        self._public_gists = None
        self._public_members_url = None
        self._public_repos = None
        self._repos_url = None
        self._total_private_repos = None
        self._twitter_username = None
        self._two_factor_requirement_enabled = None
        self._type = None
        self._updated_at = None
        self._url = None
        self.discriminator = None

        self.avatar_url = avatar_url
        self.billing_email = billing_email
        self.blog = blog
        self.collaborators = collaborators
        self.company = company
        self.created_at = created_at
        self.default_repository_permission = default_repository_permission
        self.default_repository_settings = default_repository_settings
        self.description = description
        self.disk_usage = disk_usage
        self.email = email
        self.events_url = events_url
        self.followers = followers
        self.following = following
        self.has_organization_projects = has_organization_projects
        self.has_repository_projects = has_repository_projects
        self.hooks_url = hooks_url
        self.html_url = html_url
        self.id = id
        self.is_verified = is_verified
        self.issues_url = issues_url
        self.location = location
        self.login = login
        self.members_allowed_repository_creation_type = members_allowed_repository_creation_type
        self.members_can_create_internal_repositories = members_can_create_internal_repositories
        self.members_can_create_pages = members_can_create_pages
        self.members_can_create_private_pages = members_can_create_private_pages
        self.members_can_create_private_repositories = members_can_create_private_repositories
        self.members_can_create_public_pages = members_can_create_public_pages
        self.members_can_create_public_repositories = members_can_create_public_repositories
        self.members_can_create_repositories = members_can_create_repositories
        self.members_url = members_url
        self.name = name
        self.node_id = node_id
        self.owned_private_repos = owned_private_repos
        if plan is not None:
            self.plan = plan
        self.private_gists = private_gists
        self.public_gists = public_gists
        self.public_members_url = public_members_url
        self.public_repos = public_repos
        self.repos_url = repos_url
        self.total_private_repos = total_private_repos
        self.twitter_username = twitter_username
        self.two_factor_requirement_enabled = two_factor_requirement_enabled
        self.type = type
        self.updated_at = updated_at
        self.url = url

    @property
    def avatar_url(self):
        """Gets the avatar_url of this GithubOrganization.  # noqa: E501


        :return: The avatar_url of this GithubOrganization.  # noqa: E501
        :rtype: str
        """
        return self._avatar_url

    @avatar_url.setter
    def avatar_url(self, avatar_url):
        """Sets the avatar_url of this GithubOrganization.


        :param avatar_url: The avatar_url of this GithubOrganization.  # noqa: E501
        :type avatar_url: str
        """

        self._avatar_url = avatar_url

    @property
    def billing_email(self):
        """Gets the billing_email of this GithubOrganization.  # noqa: E501


        :return: The billing_email of this GithubOrganization.  # noqa: E501
        :rtype: str
        """
        return self._billing_email

    @billing_email.setter
    def billing_email(self, billing_email):
        """Sets the billing_email of this GithubOrganization.


        :param billing_email: The billing_email of this GithubOrganization.  # noqa: E501
        :type billing_email: str
        """

        self._billing_email = billing_email

    @property
    def blog(self):
        """Gets the blog of this GithubOrganization.  # noqa: E501


        :return: The blog of this GithubOrganization.  # noqa: E501
        :rtype: str
        """
        return self._blog

    @blog.setter
    def blog(self, blog):
        """Sets the blog of this GithubOrganization.


        :param blog: The blog of this GithubOrganization.  # noqa: E501
        :type blog: str
        """

        self._blog = blog

    @property
    def collaborators(self):
        """Gets the collaborators of this GithubOrganization.  # noqa: E501


        :return: The collaborators of this GithubOrganization.  # noqa: E501
        :rtype: int
        """
        return self._collaborators

    @collaborators.setter
    def collaborators(self, collaborators):
        """Sets the collaborators of this GithubOrganization.


        :param collaborators: The collaborators of this GithubOrganization.  # noqa: E501
        :type collaborators: int
        """

        self._collaborators = collaborators

    @property
    def company(self):
        """Gets the company of this GithubOrganization.  # noqa: E501


        :return: The company of this GithubOrganization.  # noqa: E501
        :rtype: str
        """
        return self._company

    @company.setter
    def company(self, company):
        """Sets the company of this GithubOrganization.


        :param company: The company of this GithubOrganization.  # noqa: E501
        :type company: str
        """

        self._company = company

    @property
    def created_at(self):
        """Gets the created_at of this GithubOrganization.  # noqa: E501


        :return: The created_at of this GithubOrganization.  # noqa: E501
        :rtype: datetime
        """
        return self._created_at

    @created_at.setter
    def created_at(self, created_at):
        """Sets the created_at of this GithubOrganization.


        :param created_at: The created_at of this GithubOrganization.  # noqa: E501
        :type created_at: datetime
        """

        self._created_at = created_at

    @property
    def default_repository_permission(self):
        """Gets the default_repository_permission of this GithubOrganization.  # noqa: E501


        :return: The default_repository_permission of this GithubOrganization.  # noqa: E501
        :rtype: str
        """
        return self._default_repository_permission

    @default_repository_permission.setter
    def default_repository_permission(self, default_repository_permission):
        """Sets the default_repository_permission of this GithubOrganization.


        :param default_repository_permission: The default_repository_permission of this GithubOrganization.  # noqa: E501
        :type default_repository_permission: str
        """

        self._default_repository_permission = default_repository_permission

    @property
    def default_repository_settings(self):
        """Gets the default_repository_settings of this GithubOrganization.  # noqa: E501


        :return: The default_repository_settings of this GithubOrganization.  # noqa: E501
        :rtype: str
        """
        return self._default_repository_settings

    @default_repository_settings.setter
    def default_repository_settings(self, default_repository_settings):
        """Sets the default_repository_settings of this GithubOrganization.


        :param default_repository_settings: The default_repository_settings of this GithubOrganization.  # noqa: E501
        :type default_repository_settings: str
        """

        self._default_repository_settings = default_repository_settings

    @property
    def description(self):
        """Gets the description of this GithubOrganization.  # noqa: E501


        :return: The description of this GithubOrganization.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this GithubOrganization.


        :param description: The description of this GithubOrganization.  # noqa: E501
        :type description: str
        """

        self._description = description

    @property
    def disk_usage(self):
        """Gets the disk_usage of this GithubOrganization.  # noqa: E501


        :return: The disk_usage of this GithubOrganization.  # noqa: E501
        :rtype: int
        """
        return self._disk_usage

    @disk_usage.setter
    def disk_usage(self, disk_usage):
        """Sets the disk_usage of this GithubOrganization.


        :param disk_usage: The disk_usage of this GithubOrganization.  # noqa: E501
        :type disk_usage: int
        """

        self._disk_usage = disk_usage

    @property
    def email(self):
        """Gets the email of this GithubOrganization.  # noqa: E501


        :return: The email of this GithubOrganization.  # noqa: E501
        :rtype: str
        """
        return self._email

    @email.setter
    def email(self, email):
        """Sets the email of this GithubOrganization.


        :param email: The email of this GithubOrganization.  # noqa: E501
        :type email: str
        """

        self._email = email

    @property
    def events_url(self):
        """Gets the events_url of this GithubOrganization.  # noqa: E501


        :return: The events_url of this GithubOrganization.  # noqa: E501
        :rtype: str
        """
        return self._events_url

    @events_url.setter
    def events_url(self, events_url):
        """Sets the events_url of this GithubOrganization.


        :param events_url: The events_url of this GithubOrganization.  # noqa: E501
        :type events_url: str
        """

        self._events_url = events_url

    @property
    def followers(self):
        """Gets the followers of this GithubOrganization.  # noqa: E501


        :return: The followers of this GithubOrganization.  # noqa: E501
        :rtype: int
        """
        return self._followers

    @followers.setter
    def followers(self, followers):
        """Sets the followers of this GithubOrganization.


        :param followers: The followers of this GithubOrganization.  # noqa: E501
        :type followers: int
        """

        self._followers = followers

    @property
    def following(self):
        """Gets the following of this GithubOrganization.  # noqa: E501


        :return: The following of this GithubOrganization.  # noqa: E501
        :rtype: int
        """
        return self._following

    @following.setter
    def following(self, following):
        """Sets the following of this GithubOrganization.


        :param following: The following of this GithubOrganization.  # noqa: E501
        :type following: int
        """

        self._following = following

    @property
    def has_organization_projects(self):
        """Gets the has_organization_projects of this GithubOrganization.  # noqa: E501


        :return: The has_organization_projects of this GithubOrganization.  # noqa: E501
        :rtype: bool
        """
        return self._has_organization_projects

    @has_organization_projects.setter
    def has_organization_projects(self, has_organization_projects):
        """Sets the has_organization_projects of this GithubOrganization.


        :param has_organization_projects: The has_organization_projects of this GithubOrganization.  # noqa: E501
        :type has_organization_projects: bool
        """

        self._has_organization_projects = has_organization_projects

    @property
    def has_repository_projects(self):
        """Gets the has_repository_projects of this GithubOrganization.  # noqa: E501


        :return: The has_repository_projects of this GithubOrganization.  # noqa: E501
        :rtype: bool
        """
        return self._has_repository_projects

    @has_repository_projects.setter
    def has_repository_projects(self, has_repository_projects):
        """Sets the has_repository_projects of this GithubOrganization.


        :param has_repository_projects: The has_repository_projects of this GithubOrganization.  # noqa: E501
        :type has_repository_projects: bool
        """

        self._has_repository_projects = has_repository_projects

    @property
    def hooks_url(self):
        """Gets the hooks_url of this GithubOrganization.  # noqa: E501


        :return: The hooks_url of this GithubOrganization.  # noqa: E501
        :rtype: str
        """
        return self._hooks_url

    @hooks_url.setter
    def hooks_url(self, hooks_url):
        """Sets the hooks_url of this GithubOrganization.


        :param hooks_url: The hooks_url of this GithubOrganization.  # noqa: E501
        :type hooks_url: str
        """

        self._hooks_url = hooks_url

    @property
    def html_url(self):
        """Gets the html_url of this GithubOrganization.  # noqa: E501


        :return: The html_url of this GithubOrganization.  # noqa: E501
        :rtype: str
        """
        return self._html_url

    @html_url.setter
    def html_url(self, html_url):
        """Sets the html_url of this GithubOrganization.


        :param html_url: The html_url of this GithubOrganization.  # noqa: E501
        :type html_url: str
        """

        self._html_url = html_url

    @property
    def id(self):
        """Gets the id of this GithubOrganization.  # noqa: E501


        :return: The id of this GithubOrganization.  # noqa: E501
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this GithubOrganization.


        :param id: The id of this GithubOrganization.  # noqa: E501
        :type id: int
        """

        self._id = id

    @property
    def is_verified(self):
        """Gets the is_verified of this GithubOrganization.  # noqa: E501


        :return: The is_verified of this GithubOrganization.  # noqa: E501
        :rtype: bool
        """
        return self._is_verified

    @is_verified.setter
    def is_verified(self, is_verified):
        """Sets the is_verified of this GithubOrganization.


        :param is_verified: The is_verified of this GithubOrganization.  # noqa: E501
        :type is_verified: bool
        """

        self._is_verified = is_verified

    @property
    def issues_url(self):
        """Gets the issues_url of this GithubOrganization.  # noqa: E501


        :return: The issues_url of this GithubOrganization.  # noqa: E501
        :rtype: str
        """
        return self._issues_url

    @issues_url.setter
    def issues_url(self, issues_url):
        """Sets the issues_url of this GithubOrganization.


        :param issues_url: The issues_url of this GithubOrganization.  # noqa: E501
        :type issues_url: str
        """

        self._issues_url = issues_url

    @property
    def location(self):
        """Gets the location of this GithubOrganization.  # noqa: E501


        :return: The location of this GithubOrganization.  # noqa: E501
        :rtype: str
        """
        return self._location

    @location.setter
    def location(self, location):
        """Sets the location of this GithubOrganization.


        :param location: The location of this GithubOrganization.  # noqa: E501
        :type location: str
        """

        self._location = location

    @property
    def login(self):
        """Gets the login of this GithubOrganization.  # noqa: E501


        :return: The login of this GithubOrganization.  # noqa: E501
        :rtype: str
        """
        return self._login

    @login.setter
    def login(self, login):
        """Sets the login of this GithubOrganization.


        :param login: The login of this GithubOrganization.  # noqa: E501
        :type login: str
        """

        self._login = login

    @property
    def members_allowed_repository_creation_type(self):
        """Gets the members_allowed_repository_creation_type of this GithubOrganization.  # noqa: E501


        :return: The members_allowed_repository_creation_type of this GithubOrganization.  # noqa: E501
        :rtype: str
        """
        return self._members_allowed_repository_creation_type

    @members_allowed_repository_creation_type.setter
    def members_allowed_repository_creation_type(self, members_allowed_repository_creation_type):
        """Sets the members_allowed_repository_creation_type of this GithubOrganization.


        :param members_allowed_repository_creation_type: The members_allowed_repository_creation_type of this GithubOrganization.  # noqa: E501
        :type members_allowed_repository_creation_type: str
        """

        self._members_allowed_repository_creation_type = members_allowed_repository_creation_type

    @property
    def members_can_create_internal_repositories(self):
        """Gets the members_can_create_internal_repositories of this GithubOrganization.  # noqa: E501


        :return: The members_can_create_internal_repositories of this GithubOrganization.  # noqa: E501
        :rtype: bool
        """
        return self._members_can_create_internal_repositories

    @members_can_create_internal_repositories.setter
    def members_can_create_internal_repositories(self, members_can_create_internal_repositories):
        """Sets the members_can_create_internal_repositories of this GithubOrganization.


        :param members_can_create_internal_repositories: The members_can_create_internal_repositories of this GithubOrganization.  # noqa: E501
        :type members_can_create_internal_repositories: bool
        """

        self._members_can_create_internal_repositories = members_can_create_internal_repositories

    @property
    def members_can_create_pages(self):
        """Gets the members_can_create_pages of this GithubOrganization.  # noqa: E501


        :return: The members_can_create_pages of this GithubOrganization.  # noqa: E501
        :rtype: bool
        """
        return self._members_can_create_pages

    @members_can_create_pages.setter
    def members_can_create_pages(self, members_can_create_pages):
        """Sets the members_can_create_pages of this GithubOrganization.


        :param members_can_create_pages: The members_can_create_pages of this GithubOrganization.  # noqa: E501
        :type members_can_create_pages: bool
        """

        self._members_can_create_pages = members_can_create_pages

    @property
    def members_can_create_private_pages(self):
        """Gets the members_can_create_private_pages of this GithubOrganization.  # noqa: E501


        :return: The members_can_create_private_pages of this GithubOrganization.  # noqa: E501
        :rtype: bool
        """
        return self._members_can_create_private_pages

    @members_can_create_private_pages.setter
    def members_can_create_private_pages(self, members_can_create_private_pages):
        """Sets the members_can_create_private_pages of this GithubOrganization.


        :param members_can_create_private_pages: The members_can_create_private_pages of this GithubOrganization.  # noqa: E501
        :type members_can_create_private_pages: bool
        """

        self._members_can_create_private_pages = members_can_create_private_pages

    @property
    def members_can_create_private_repositories(self):
        """Gets the members_can_create_private_repositories of this GithubOrganization.  # noqa: E501


        :return: The members_can_create_private_repositories of this GithubOrganization.  # noqa: E501
        :rtype: bool
        """
        return self._members_can_create_private_repositories

    @members_can_create_private_repositories.setter
    def members_can_create_private_repositories(self, members_can_create_private_repositories):
        """Sets the members_can_create_private_repositories of this GithubOrganization.


        :param members_can_create_private_repositories: The members_can_create_private_repositories of this GithubOrganization.  # noqa: E501
        :type members_can_create_private_repositories: bool
        """

        self._members_can_create_private_repositories = members_can_create_private_repositories

    @property
    def members_can_create_public_pages(self):
        """Gets the members_can_create_public_pages of this GithubOrganization.  # noqa: E501


        :return: The members_can_create_public_pages of this GithubOrganization.  # noqa: E501
        :rtype: bool
        """
        return self._members_can_create_public_pages

    @members_can_create_public_pages.setter
    def members_can_create_public_pages(self, members_can_create_public_pages):
        """Sets the members_can_create_public_pages of this GithubOrganization.


        :param members_can_create_public_pages: The members_can_create_public_pages of this GithubOrganization.  # noqa: E501
        :type members_can_create_public_pages: bool
        """

        self._members_can_create_public_pages = members_can_create_public_pages

    @property
    def members_can_create_public_repositories(self):
        """Gets the members_can_create_public_repositories of this GithubOrganization.  # noqa: E501


        :return: The members_can_create_public_repositories of this GithubOrganization.  # noqa: E501
        :rtype: bool
        """
        return self._members_can_create_public_repositories

    @members_can_create_public_repositories.setter
    def members_can_create_public_repositories(self, members_can_create_public_repositories):
        """Sets the members_can_create_public_repositories of this GithubOrganization.


        :param members_can_create_public_repositories: The members_can_create_public_repositories of this GithubOrganization.  # noqa: E501
        :type members_can_create_public_repositories: bool
        """

        self._members_can_create_public_repositories = members_can_create_public_repositories

    @property
    def members_can_create_repositories(self):
        """Gets the members_can_create_repositories of this GithubOrganization.  # noqa: E501


        :return: The members_can_create_repositories of this GithubOrganization.  # noqa: E501
        :rtype: bool
        """
        return self._members_can_create_repositories

    @members_can_create_repositories.setter
    def members_can_create_repositories(self, members_can_create_repositories):
        """Sets the members_can_create_repositories of this GithubOrganization.


        :param members_can_create_repositories: The members_can_create_repositories of this GithubOrganization.  # noqa: E501
        :type members_can_create_repositories: bool
        """

        self._members_can_create_repositories = members_can_create_repositories

    @property
    def members_url(self):
        """Gets the members_url of this GithubOrganization.  # noqa: E501


        :return: The members_url of this GithubOrganization.  # noqa: E501
        :rtype: str
        """
        return self._members_url

    @members_url.setter
    def members_url(self, members_url):
        """Sets the members_url of this GithubOrganization.


        :param members_url: The members_url of this GithubOrganization.  # noqa: E501
        :type members_url: str
        """

        self._members_url = members_url

    @property
    def name(self):
        """Gets the name of this GithubOrganization.  # noqa: E501


        :return: The name of this GithubOrganization.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this GithubOrganization.


        :param name: The name of this GithubOrganization.  # noqa: E501
        :type name: str
        """

        self._name = name

    @property
    def node_id(self):
        """Gets the node_id of this GithubOrganization.  # noqa: E501


        :return: The node_id of this GithubOrganization.  # noqa: E501
        :rtype: str
        """
        return self._node_id

    @node_id.setter
    def node_id(self, node_id):
        """Sets the node_id of this GithubOrganization.


        :param node_id: The node_id of this GithubOrganization.  # noqa: E501
        :type node_id: str
        """

        self._node_id = node_id

    @property
    def owned_private_repos(self):
        """Gets the owned_private_repos of this GithubOrganization.  # noqa: E501


        :return: The owned_private_repos of this GithubOrganization.  # noqa: E501
        :rtype: int
        """
        return self._owned_private_repos

    @owned_private_repos.setter
    def owned_private_repos(self, owned_private_repos):
        """Sets the owned_private_repos of this GithubOrganization.


        :param owned_private_repos: The owned_private_repos of this GithubOrganization.  # noqa: E501
        :type owned_private_repos: int
        """

        self._owned_private_repos = owned_private_repos

    @property
    def plan(self):
        """Gets the plan of this GithubOrganization.  # noqa: E501


        :return: The plan of this GithubOrganization.  # noqa: E501
        :rtype: GithubPlan
        """
        return self._plan

    @plan.setter
    def plan(self, plan):
        """Sets the plan of this GithubOrganization.


        :param plan: The plan of this GithubOrganization.  # noqa: E501
        :type plan: GithubPlan
        """

        self._plan = plan

    @property
    def private_gists(self):
        """Gets the private_gists of this GithubOrganization.  # noqa: E501


        :return: The private_gists of this GithubOrganization.  # noqa: E501
        :rtype: int
        """
        return self._private_gists

    @private_gists.setter
    def private_gists(self, private_gists):
        """Sets the private_gists of this GithubOrganization.


        :param private_gists: The private_gists of this GithubOrganization.  # noqa: E501
        :type private_gists: int
        """

        self._private_gists = private_gists

    @property
    def public_gists(self):
        """Gets the public_gists of this GithubOrganization.  # noqa: E501


        :return: The public_gists of this GithubOrganization.  # noqa: E501
        :rtype: int
        """
        return self._public_gists

    @public_gists.setter
    def public_gists(self, public_gists):
        """Sets the public_gists of this GithubOrganization.


        :param public_gists: The public_gists of this GithubOrganization.  # noqa: E501
        :type public_gists: int
        """

        self._public_gists = public_gists

    @property
    def public_members_url(self):
        """Gets the public_members_url of this GithubOrganization.  # noqa: E501


        :return: The public_members_url of this GithubOrganization.  # noqa: E501
        :rtype: str
        """
        return self._public_members_url

    @public_members_url.setter
    def public_members_url(self, public_members_url):
        """Sets the public_members_url of this GithubOrganization.


        :param public_members_url: The public_members_url of this GithubOrganization.  # noqa: E501
        :type public_members_url: str
        """

        self._public_members_url = public_members_url

    @property
    def public_repos(self):
        """Gets the public_repos of this GithubOrganization.  # noqa: E501


        :return: The public_repos of this GithubOrganization.  # noqa: E501
        :rtype: int
        """
        return self._public_repos

    @public_repos.setter
    def public_repos(self, public_repos):
        """Sets the public_repos of this GithubOrganization.


        :param public_repos: The public_repos of this GithubOrganization.  # noqa: E501
        :type public_repos: int
        """

        self._public_repos = public_repos

    @property
    def repos_url(self):
        """Gets the repos_url of this GithubOrganization.  # noqa: E501


        :return: The repos_url of this GithubOrganization.  # noqa: E501
        :rtype: str
        """
        return self._repos_url

    @repos_url.setter
    def repos_url(self, repos_url):
        """Sets the repos_url of this GithubOrganization.


        :param repos_url: The repos_url of this GithubOrganization.  # noqa: E501
        :type repos_url: str
        """

        self._repos_url = repos_url

    @property
    def total_private_repos(self):
        """Gets the total_private_repos of this GithubOrganization.  # noqa: E501


        :return: The total_private_repos of this GithubOrganization.  # noqa: E501
        :rtype: int
        """
        return self._total_private_repos

    @total_private_repos.setter
    def total_private_repos(self, total_private_repos):
        """Sets the total_private_repos of this GithubOrganization.


        :param total_private_repos: The total_private_repos of this GithubOrganization.  # noqa: E501
        :type total_private_repos: int
        """

        self._total_private_repos = total_private_repos

    @property
    def twitter_username(self):
        """Gets the twitter_username of this GithubOrganization.  # noqa: E501


        :return: The twitter_username of this GithubOrganization.  # noqa: E501
        :rtype: str
        """
        return self._twitter_username

    @twitter_username.setter
    def twitter_username(self, twitter_username):
        """Sets the twitter_username of this GithubOrganization.


        :param twitter_username: The twitter_username of this GithubOrganization.  # noqa: E501
        :type twitter_username: str
        """

        self._twitter_username = twitter_username

    @property
    def two_factor_requirement_enabled(self):
        """Gets the two_factor_requirement_enabled of this GithubOrganization.  # noqa: E501


        :return: The two_factor_requirement_enabled of this GithubOrganization.  # noqa: E501
        :rtype: bool
        """
        return self._two_factor_requirement_enabled

    @two_factor_requirement_enabled.setter
    def two_factor_requirement_enabled(self, two_factor_requirement_enabled):
        """Sets the two_factor_requirement_enabled of this GithubOrganization.


        :param two_factor_requirement_enabled: The two_factor_requirement_enabled of this GithubOrganization.  # noqa: E501
        :type two_factor_requirement_enabled: bool
        """

        self._two_factor_requirement_enabled = two_factor_requirement_enabled

    @property
    def type(self):
        """Gets the type of this GithubOrganization.  # noqa: E501


        :return: The type of this GithubOrganization.  # noqa: E501
        :rtype: str
        """
        return self._type

    @type.setter
    def type(self, type):
        """Sets the type of this GithubOrganization.


        :param type: The type of this GithubOrganization.  # noqa: E501
        :type type: str
        """

        self._type = type

    @property
    def updated_at(self):
        """Gets the updated_at of this GithubOrganization.  # noqa: E501


        :return: The updated_at of this GithubOrganization.  # noqa: E501
        :rtype: datetime
        """
        return self._updated_at

    @updated_at.setter
    def updated_at(self, updated_at):
        """Sets the updated_at of this GithubOrganization.


        :param updated_at: The updated_at of this GithubOrganization.  # noqa: E501
        :type updated_at: datetime
        """

        self._updated_at = updated_at

    @property
    def url(self):
        """Gets the url of this GithubOrganization.  # noqa: E501


        :return: The url of this GithubOrganization.  # noqa: E501
        :rtype: str
        """
        return self._url

    @url.setter
    def url(self, url):
        """Sets the url of this GithubOrganization.


        :param url: The url of this GithubOrganization.  # noqa: E501
        :type url: str
        """

        self._url = url

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, GithubOrganization):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, GithubOrganization):
            return True

        return self.to_dict() != other.to_dict()



from typing import List, Tuple

import click

from vessl.cli._base import VesslGroup, vessl_argument, vessl_option
from vessl.cli._util import (
    Endpoint,
    generic_prompter,
    print_data,
    print_logs,
    print_table,
    print_volume_files,
    prompt_choices,
    truncate_datetime,
)
from vessl.cli.organization import organization_name_option
from vessl.cli.project import project_name_option
from vessl.experiment import (
    create_experiment,
    download_experiment_output_files,
    list_experiment_logs,
    list_experiment_output_files,
    list_experiments,
    read_experiment,
)
from vessl.kernel_cluster import list_cluster_nodes, list_clusters, read_cluster
from vessl.kernel_image import list_kernel_images
from vessl.kernel_resource_spec import list_kernel_resource_specs
from vessl.util.constant import (
    MOUNT_PATH_OUTPUT,
    PROCESSOR_TYPE_GPU,
    PROCESSOR_TYPES,
)


def experiment_name_prompter(
    ctx: click.Context, param: click.Parameter, value: str
) -> str:
    experiments = list_experiments()
    return prompt_choices(
        "Experiment", [(f"{x.name} #{x.number}", x.name) for x in reversed(experiments)]
    )


def cluster_name_prompter(
    ctx: click.Context, param: click.Parameter, value: str
) -> str:
    clusters = list_clusters()
    cluster = prompt_choices(
        "Cluster",
        [
            (f"{x.name}{' (Custom)' if not x.is_savvihub_managed else ''}", x)
            for x in clusters
        ],
    )
    ctx.obj["cluster"] = cluster
    return cluster.name


def cluster_name_callback(
    ctx: click.Context, param: click.Parameter, value: str
) -> str:
    if "cluster" not in ctx.obj:
        ctx.obj["cluster"] = read_cluster(value)
    return value


def resource_name_prompter(
    ctx: click.Context, param: click.Parameter, value: str
) -> str:
    cluster = ctx.obj.get("cluster")
    if cluster is None:
        raise click.BadOptionUsage(
            option_name="--cluster",
            message="cluster (`--cluster`) must first be specified.",
        )

    if cluster.is_savvihub_managed:
        resources = list_kernel_resource_specs()
        resource = prompt_choices("Resource", [(x.name, x) for x in resources])
        ctx.obj["processor_type"] = resource.processor_type
        return resource.name


def processor_type_prompter(
    ctx: click.Context, param: click.Parameter, value: str
) -> str:
    cluster = ctx.obj.get("cluster")
    if cluster is None:
        raise click.BadOptionUsage(
            option_name="--cluster",
            message="cluster (`--cluster`) must first be specified.",
        )

    if not cluster.is_savvihub_managed:
        processor_type = prompt_choices("Processor Type", PROCESSOR_TYPES)
        ctx.obj["processor_type"] = processor_type
        return processor_type


def cpu_limit_prompter(
    ctx: click.Context, param: click.Parameter, value: float
) -> float:
    cluster = ctx.obj.get("cluster")
    if cluster is None:
        raise click.BadOptionUsage(
            option_name="--cluster",
            message="cluster (`--cluster`) must first be specified.",
        )

    if not cluster.is_savvihub_managed:
        return click.prompt("CPUs (in vCPU)", type=click.FLOAT)


def memory_limit_prompter(
    ctx: click.Context, param: click.Parameter, value: float
) -> float:
    cluster = ctx.obj.get("cluster")
    if cluster is None:
        raise click.BadOptionUsage(
            option_name="--cluster",
            message="cluster (`--cluster`) must first be specified.",
        )

    if not cluster.is_savvihub_managed:
        return click.prompt("Memory (in GiB)", type=click.FLOAT)


def gpu_type_prompter(ctx: click.Context, param: click.Parameter, value: str) -> str:
    cluster = ctx.obj.get("cluster")
    if cluster is None:
        raise click.BadOptionUsage(
            option_name="--cluster",
            message="cluster (`--cluster`) must first be specified.",
        )

    processor_type = ctx.obj.get("processor_type")
    if processor_type is None:
        raise click.UsageError(
            message="Processor type must first be specified.",
        )

    if not cluster.is_savvihub_managed and processor_type == PROCESSOR_TYPE_GPU:
        nodes = list_cluster_nodes(cluster.name)
        return prompt_choices("GPU Type", [x.gpu_product_name for x in nodes])


def gpu_limit_prompter(
    ctx: click.Context, param: click.Parameter, value: float
) -> float:
    cluster = ctx.obj.get("cluster")
    if cluster is None:
        raise click.BadOptionUsage(
            option_name="--cluster",
            message="cluster (`--cluster`) must first be specified.",
        )

    processor_type = ctx.obj.get("processor_type")
    if processor_type is None:
        raise click.UsageError(
            message="Processor type must first be specified.",
        )

    if not cluster.is_savvihub_managed and processor_type == PROCESSOR_TYPE_GPU:
        return click.prompt("GPUs (in vGPU)", type=click.FLOAT)


def image_url_prompter(ctx: click.Context, param: click.Parameter, value: str) -> str:
    processor_type = ctx.obj.get("processor_type")
    if processor_type is None:
        raise click.UsageError(
            message="Processor type must first be specified.",
        )

    images = list_kernel_images()
    images = [x for x in images if x.processor_type == processor_type]

    return prompt_choices("Image URL", [x.image_url for x in images])


@click.command(name="experiment", cls=VesslGroup)
def cli():
    pass


@cli.vessl_command()
@vessl_argument(
    "name", type=click.STRING, required=True, prompter=experiment_name_prompter
)
@organization_name_option
@project_name_option
def read(name: str):
    experiment = read_experiment(experiment_name_or_number=name)
    print_data(
        {
            "ID": experiment.id,
            "Number": experiment.number,
            "Name": experiment.name,
            "Status": experiment.status,
            "Created": truncate_datetime(experiment.created_dt),
            "Message": experiment.message,
            "Source Code": experiment.source_code_link[0].url,
            "Kernel Image": {
                "Name": experiment.kernel_image.name,
                "URL": experiment.kernel_image.image_url,
            },
            "Resource Spec": {
                "Name": experiment.kernel_resource_spec.name,
                "CPU Type": experiment.kernel_resource_spec.cpu_type,
                "CPU Limit": experiment.kernel_resource_spec.cpu_limit,
                "Memory Limit": experiment.kernel_resource_spec.memory_limit,
                "GPU Type": experiment.kernel_resource_spec.gpu_type,
                "GPU Limit": experiment.kernel_resource_spec.gpu_limit,
            },
            "Start command": experiment.start_command,
        }
    )
    print(
        f"For more info: {Endpoint.experiment.format(experiment.organization.name, experiment.project.name, experiment.number)}"
    )


@cli.vessl_command()
@organization_name_option
@project_name_option
def list():
    experiments = list_experiments()
    print_table(
        experiments,
        ["ID", "Number", "Name", "Status", "Created", "Message"],
        lambda x: [
            x.id,
            x.number,
            x.name,
            x.status,
            truncate_datetime(x.created_dt),
            x.message,
        ],
    )


cluster_option = vessl_option(
    "-c",
    "--cluster",
    type=click.STRING,
    required=True,
    prompter=cluster_name_prompter,
    callback=cluster_name_callback,
    help="Must be specified before resource-related options (`--resource`, `--processor`, ...).",
)
command_option = vessl_option(
    "-x",
    "--command",
    type=click.STRING,
    required=True,
    prompter=generic_prompter("Start command"),
    help="Start command to execute in experiment container.",
)
resource_option = vessl_option(
    "-r",
    "--resource",
    type=click.STRING,
    prompter=resource_name_prompter,
    help="Resource type to run experiment (for managed cluster only).",
)
processor_option = vessl_option(
    "--processor",
    type=click.Choice(("cpu", "gpu")),
    prompter=processor_type_prompter,
    help="CPU or GPU (for custom cluster only).",
)
cpu_limit_option = vessl_option(
    "--cpu-limit",
    type=click.FLOAT,
    prompter=cpu_limit_prompter,
    help="Number of vCPUs (for custom cluster only).",
)
memory_limit_option = vessl_option(
    "--memory-limit",
    type=click.FLOAT,
    prompter=memory_limit_prompter,
    help="Memory limit in GiB (for custom cluster only).",
)
gpu_type_option = vessl_option(
    "--gpu-type",
    type=click.STRING,
    prompter=gpu_type_prompter,
    help="GPU type such as Tesla-K80 (for custom cluster only).",
)
gpu_limit_option = vessl_option(
    "--gpu-limit",
    type=click.FLOAT,
    prompter=gpu_limit_prompter,
    help="Number of GPU cores (for custom cluster only).",
)
image_url_option = vessl_option(
    "-i",
    "--image-url",
    type=click.STRING,
    prompter=image_url_prompter,
    help="Kernel docker image URL",
)
message_option = click.option("-m", "--message", type=click.STRING)
termination_protection_option = click.option("--termination-protection", is_flag=True)
env_var_option = click.option(
    "-e",
    "--env-var",
    type=click.Tuple([str, str]),
    multiple=True,
    help="Environment variables. Format: [key] [value], ex. `--env-var PORT 8080`.",
)
dataset_option = click.option(
    "--dataset",
    type=click.Tuple([str, str]),
    multiple=True,
    help="Dataset mounts. Format: [mount_path] [dataset_name], ex. `--dataset /input mnist`.",
)
root_volume_size_option = click.option("--root-volume-size", type=click.STRING)
working_dir_option = click.option(
    "--working-dir", type=click.STRING, help="Defaults to `/work/[project_name]`."
)
output_dir_option = click.option(
    "--output-dir",
    type=click.STRING,
    default=MOUNT_PATH_OUTPUT,
    help="Directory to store experiment output files. Defaults to `/output`.",
)
local_project_option = click.option(
    "--local-project", type=click.STRING, help="Local project file URL"
)


@cli.vessl_command()
@cluster_option
@command_option
@resource_option
@processor_option
@cpu_limit_option
@memory_limit_option
@gpu_type_option
@gpu_limit_option
@image_url_option
@message_option
@termination_protection_option
@env_var_option
@dataset_option
@root_volume_size_option
@working_dir_option
@output_dir_option
@local_project_option
@organization_name_option
@project_name_option
def create(
    cluster: str,
    command: str,
    resource: str,
    processor: str,
    cpu_limit: float,
    memory_limit: float,
    gpu_type: str,
    gpu_limit: int,
    image_url: str,
    message: str,
    termination_protection: bool,
    env_var: List[Tuple[str, str]],
    dataset: List[Tuple[str, str]],
    root_volume_size: str,
    working_dir: str,
    output_dir: str,
    local_project: str,
):
    experiment = create_experiment(
        cluster_name=cluster,
        start_command=command,
        kernel_resource_spec_name=resource,
        processor_type=processor,
        cpu_limit=cpu_limit,
        memory_limit=memory_limit,
        gpu_type=gpu_type,
        gpu_limit=gpu_limit,
        kernel_image_url=image_url,
        message=message,
        termination_protection=termination_protection,
        env_vars=env_var,
        dataset_mounts=dataset,
        root_volume_size=root_volume_size,
        working_dir=working_dir,
        output_dir=output_dir,
        local_project_url=local_project,
    )
    print(
        f"Created '{experiment.name}'.\n"
        f"For more info: {Endpoint.experiment.format(experiment.organization.name, experiment.project.name, experiment.number)}"
    )


@cli.vessl_command()
@vessl_argument(
    "name", type=click.STRING, required=True, prompter=experiment_name_prompter
)
@click.option(
    "--tail",
    type=click.INT,
    default=200,
    help="Number of lines to display (from the end).",
)
@organization_name_option
@project_name_option
def logs(name: str, tail: int):
    logs = list_experiment_logs(experiment_name=name, tail=tail)
    print_logs(logs)
    print(f"Displayed last {len(logs)} lines of '{name}'.")


@cli.vessl_command()
@vessl_argument(
    "name", type=click.STRING, required=True, prompter=experiment_name_prompter
)
@click.option("-r", "--recursive", is_flag=True)
@organization_name_option
@project_name_option
def list_output(
    name: str,
    recursive: bool,
):
    files = list_experiment_output_files(
        experiment_name=name,
        need_download_url=False,
        recursive=recursive,
    )
    print_volume_files(files)


@cli.vessl_command()
@vessl_argument(
    "name", type=click.STRING, required=True, prompter=experiment_name_prompter
)
@click.option(
    "-p",
    "--path",
    type=click.Path(),
    default="./output",
    help="Path to store downloads. Defaults to `./output`.",
)
@organization_name_option
@project_name_option
def download_output(name: str, path: str):
    download_experiment_output_files(
        experiment_name=name,
        dest_path=path,
    )
    print(f"Downloaded experiment to {path}.")



import os
from pathlib import Path

VESSL_LOG_LEVEL_DEBUG = "DEBUG"
VESSL_LOG_LEVEL_INFO = "INFO"
VESSL_LOG_LEVEL_WARNING = "WARNING"
VESSL_LOG_LEVEL_ERROR = "ERROR"
VESSL_LOG_LEVEL_LEVELS = [
    VESSL_LOG_LEVEL_DEBUG,
    VESSL_LOG_LEVEL_INFO,
    VESSL_LOG_LEVEL_WARNING,
    VESSL_LOG_LEVEL_ERROR,
]

VESSL_LOG_LEVEL = (
    os.environ.get("VESSL_LOG")
    if os.environ.get("VESSL_LOG") in VESSL_LOG_LEVEL_LEVELS
    else VESSL_LOG_LEVEL_WARNING
)
WEB_HOST = os.environ.get("VESSL_WEB_HOST", "https://vessl.ai")
API_HOST = os.environ.get("VESSL_API_HOST", "https://api.vessl.ai")

LOGIN_TIMEOUT_SECONDS = 160
ACCESS_TOKEN_ENV_VAR = "VESSL_ACCESS_TOKEN"
DEFAULT_ORGANIZATION_ENV_VAR = "VESSL_DEFAULT_ORGANIZATION"
CREDENTIALS_FILE_ENV_VAR = "VESSL_CREDENTIALS_FILE"

PROJECT_TYPE_VERSION_CONTROL = "version-control"
PROJECT_TYPE_CLI_DRIVEN = "cli-driven"
PROJECT_TYPES = [PROJECT_TYPE_VERSION_CONTROL, PROJECT_TYPE_CLI_DRIVEN]

DATASET_PATH_SCHEME_S3 = "s3://"
DATASET_PATH_SCHEME_GS = "gs://"

DATASET_VERSION_HASH_LATEST = "latest"

PROCESSOR_TYPE_CPU = "CPU"
PROCESSOR_TYPE_GPU = "GPU"
PROCESSOR_TYPES = [PROCESSOR_TYPE_CPU, PROCESSOR_TYPE_GPU]

SWEEP_OBJECTIVE_TYPE_MAXIMIZE = "maximize"
SWEEP_OBJECTIVE_TYPE_MINIMIZE = "minimize"
SWEEP_OBJECTIVE_TYPES = [SWEEP_OBJECTIVE_TYPE_MAXIMIZE, SWEEP_OBJECTIVE_TYPE_MINIMIZE]

SWEEP_ALGORITHM_TYPE_GRID = "grid"
SWEEP_ALGORITHM_TYPE_RANDOM = "random"
SWEEP_ALGORITHM_TYPE_BAYESIAN = "baysian"
SWEEP_ALGORITHM_TYPES = [
    SWEEP_ALGORITHM_TYPE_GRID,
    SWEEP_ALGORITHM_TYPE_RANDOM,
    SWEEP_ALGORITHM_TYPE_BAYESIAN,
]

SWEEP_PARAMETER_TYPE_CATEGORICAL = "categorical"
SWEEP_PARAMETER_TYPE_INT = "int"
SWEEP_PARAMETER_TYPE_DOUBLE = "double"
SWEEP_PARAMETER_TYPES = [
    SWEEP_PARAMETER_TYPE_CATEGORICAL,
    SWEEP_PARAMETER_TYPE_INT,
    SWEEP_PARAMETER_TYPE_DOUBLE,
]

SWEEP_PARAMETER_RANGE_TYPE_SPACE = "space"
SWEEP_PARAMETER_RANGE_TYPE_LIST = "list"
SWEEP_PARAMETER_RANGE_TYPES = [
    SWEEP_PARAMETER_RANGE_TYPE_SPACE,
    SWEEP_PARAMETER_RANGE_TYPE_LIST,
]

MOUNT_TYPE_DATASET = "dataset"
MOUNT_TYPE_DATASET_VERSION = "dataset-version"
MOUNT_TYPE_EMPTY_DIR = "empty-dir"
MOUNT_TYPE_OUTPUT = "output"
MOUNT_TYPE_PROJECT = "project"

MOUNT_PATH_EMPTY_DIR = "/work/"
MOUNT_PATH_OUTPUT = "/output/"
MOUNT_PATH_PROJECT = "/work/{}"

PARALLEL_WORKERS = os.environ.get("VESSL_PARALLEL_WORKERS", 20)

VESSL_IMAGE_PATH = "vessl-images"
VESSL_PLOTS_FILETYPE_IMAGE = "image"
VESSL_PLOTS_FILETYPE_IMAGES = "images"

WORKSPACE_ZIP_PATH = "workspace-backup.zip"
WORKSPACE_TEMP_ZIP_PATH = f"/tmp/{WORKSPACE_ZIP_PATH}"

SSH_CONFIG_PATH = f"{Path.home()}/.ssh/config"
SSH_CONFIG_FORMAT = """Host {host}
    User vessl
    Hostname {hostname}
    Port {port}
    StrictHostKeyChecking accept-new
    CheckHostIP no
"""
SSH_PUBLIC_KEY_PATH = f"{Path.home()}/.ssh/id_ed25519.pub"
SSH_PRIVATE_KEY_PATH = f"{Path.home()}/.ssh/id_ed25519"



import click

from vessl.cli._base import VesslGroup, vessl_argument
from vessl.cli._util import format_bool, print_data, print_table, prompt_choices
from vessl.cli.organization import organization_name_option
from vessl.kernel_resource_spec import (
    list_kernel_resource_specs,
    read_kernel_resource_spec,
)


def resource_name_prompter(
    ctx: click.Context, param: click.Parameter, value: str
) -> str:
    resources = list_kernel_resource_specs()
    return prompt_choices("Resource", [x.name for x in resources])


@click.command(name="resource", cls=VesslGroup)
def cli():
    pass


@cli.vessl_command()
@vessl_argument(
    "name", type=click.STRING, required=True, prompter=resource_name_prompter
)
@organization_name_option
def read(name: str):
    resource = read_kernel_resource_spec(kernel_resource_spec_name=name)
    print_data(
        {
            "ID": resource.id,
            "Name": resource.name,
            "Description": resource.description,
            "CPU Type": resource.cpu_type,
            "CPU Limit": resource.cpu_limit,
            "GPU Type": resource.gpu_type,
            "GPU Limit": resource.gpu_limit,
            "Memory Limit": resource.memory_limit,
            "Spot": format_bool(resource.spot),
            "Region": resource.region,
        }
    )


@cli.vessl_command()
@organization_name_option
def list():
    resources = list_kernel_resource_specs()
    print_table(
        resources,
        [
            "ID",
            "Name",
            "CPU Type",
            "CPU Limit",
            "GPU Type",
            "GPU Limit",
            "Memory Limit",
            "Spot",
        ],
        lambda x: [
            x.id,
            x.name,
            x.cpu_type,
            x.cpu_limit,
            x.gpu_type,
            x.gpu_limit,
            x.memory_limit,
            x.spot,
        ],
    )



import subprocess
from typing import List

from openapi_client.models import (
    CLIProjectCreateAPIPayload,
    GitHubProjectCreateAPIPayload,
    ResponseProjectInfo,
)
from vessl import vessl_api
from vessl.organization import _get_organization_name
from vessl.util import logger
from vessl.util.constant import PROJECT_TYPE_CLI_DRIVEN, PROJECT_TYPE_VERSION_CONTROL
from vessl.util.exception import InvalidProjectError


def read_project(project_name: str, **kwargs) -> ResponseProjectInfo:
    """Read project

    Keyword args:
        organization_name (str): override default organization
    """
    return vessl_api.project_read_api(
        organization_name=_get_organization_name(**kwargs),
        project_name=project_name,
    )


def list_projects(**kwargs) -> List[ResponseProjectInfo]:
    """List projects

    Keyword args:
        organization_name (str): override default organization
    """
    return vessl_api.project_list_api(
        organization_name=_get_organization_name(**kwargs),
    ).results


def create_project(
    project_type: str,
    project_name: str,
    description: str = None,
    github_owner: str = None,
    github_repo: str = None,
    **kwargs,
) -> ResponseProjectInfo:
    if project_type == PROJECT_TYPE_VERSION_CONTROL:
        return vessl_api.git_hub_project_create_api(
            organization_name=_get_organization_name(**kwargs),
            git_hub_project_create_api_payload=GitHubProjectCreateAPIPayload(
                name=project_name,
                description=description,
                github_owner=github_owner,
                github_repo=github_repo,
            ),
        )

    if project_type == PROJECT_TYPE_CLI_DRIVEN:
        return vessl_api.c_li_project_create_api(
            organization_name=_get_organization_name(**kwargs),
            cli_project_create_api_payload=CLIProjectCreateAPIPayload(
                name=project_name,
                description=description,
            ),
        )

    raise InvalidProjectError(
        f"Invalid project type: {project_type}. Must be version-control or cli-driven."
    )


def clone_project(project_name: str, **kwargs) -> None:
    """Clone project

    Keyword args:
        organization_name (str): override default organization
    """
    project = read_project(project_name, **kwargs)

    if project.type != PROJECT_TYPE_VERSION_CONTROL:
        raise InvalidProjectError("Project type must be version-control.")

    github_token = vessl_api.cli_git_hub_token_api().token

    dirname = project.cached_git_repo_slug
    git_url = f"https://{github_token}@github.com/{project.cached_git_owner_slug}/{project.cached_git_repo_slug}.git"
    try:
        subprocess.check_output(["git", "clone", git_url])
    except subprocess.CalledProcessError:
        dirname = f"savvihub-{project.cached_git_repo_slug}"
        logger.info(f"Falling back to '{dirname}'...")
        subprocess.check_output(["git", "clone", git_url, dirname])

    subprocess.check_output(["rm", "-rf", f"{dirname}/.git"])


def _get_project_name(**kwargs) -> str:
    project_name = kwargs.get("project_name")
    if project_name is not None:
        return project_name
    if vessl_api.project is not None:
        return vessl_api.project.name
    raise InvalidProjectError("No project selected.")


def _get_project(**kwargs) -> ResponseProjectInfo:
    project_name = kwargs.get("project_name")
    if project_name is not None:
        return read_project(project_name)
    if vessl_api.project is not None:
        return vessl_api.project
    raise InvalidProjectError("No project selected.")
